from data_gateway_sdk.protocols.smpm.smpm_ul_device_energy_16b_retrospective_energy import SmpmUlDeviceEnergy16BRetrospectiveEnergyData, SmpmUlDeviceEnergy16bRetrospectiveEnergyIds
from data_gateway_sdk.utils.buf_ref import BufRef


def test_smpm_ul_device_energy_16b_retrospective_energy() -> None:
    case_serialized = bytes.fromhex("90f3188e67432784000000c0eebd0b00")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=False, period_ago=15, value_current=285591.16, value_previous_1_delta=5413.0, value_previous_2_delta=0.0, value_previous_3_delta=123123.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=False, period_ago=15, value_current=4312123.0, value_previous_1_delta=5413.0, value_previous_2_delta=0.0, value_previous_3_delta=123123.0))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("900bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("90fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.DAILY_ENERGY_ACTIVE_CONSUMED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10b0000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a10bfeffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=0, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=0, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fb0100000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=0.0, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=0.0, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=0.0, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=0.0, value_previous_3_delta=335544.32))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=0.0))  # noqa: E501
    case_serialized = bytes.fromhex("a1fbffffff0f00000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BRetrospectiveEnergyData(packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, is_valid=True, period_ago=31, value_current=1342177.27, value_previous_1_delta=0.0, value_previous_2_delta=0.0, value_previous_3_delta=0.0) == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BRetrospectiveEnergyData.serialize(SmpmUlDeviceEnergy16BRetrospectiveEnergyData(is_valid=True, packet_type_id_enum=SmpmUlDeviceEnergy16bRetrospectiveEnergyIds.MONTHLY_ENERGY_REACTIVE_GENERATED, period_ago=31, value_current=2684354.55, value_previous_1_delta=335544.32, value_previous_2_delta=335544.32, value_previous_3_delta=335544.32))  # noqa: E501



from typing import Any, Dict


class XteaError(Exception):
    def __init__(self, massage: str, *args: Any, **kwargs: Dict[str, Any]) -> None:
        super(*args, **kwargs)
        self.message = massage


class AesError(Exception):
    def __init__(self, massage: str, *args: Any, **kwargs: Dict[str, Any]) -> None:
        super(*args, **kwargs)
        self.message = massage


class KuznechikError(Exception):
    def __init__(self, massage: str, *args: Any, **kwargs: Dict[str, Any]) -> None:
        super(*args, **kwargs)
        self.message = massage



from datetime import timedelta

from data_gateway_sdk.protocols.smpm.smpm_ul_device_energy_16b_status_info import SmpmUlDeviceEnergy16BStatusInfoData
from data_gateway_sdk.utils.buf_ref import BufRef


def test_smpm_ul_device_energy_16b_status_info() -> None:
    case_serialized = bytes.fromhex("bd03f8ac0e013eabc3d05e0000000000")
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=567648000.0), uptime_s=timedelta(seconds=567648000.0), battery_volts=3.3, temperature=23, relay_switch=1, relay_is_active=False) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=567648000.0), uptime_s=timedelta(seconds=567648000.0), battery_volts=3.3, temperature=23, relay_switch=1, relay_is_active=False))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000000c03f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000000c03f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd030000000000000000000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd030000000000000000000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000000c0ff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000000c0ff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd030000000000000000c00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd030000000000000000c00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000000c03f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000000c03f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff0100000000000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff0100000000000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000000c0ff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000000c0ff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff0100000000c00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff0100000000c00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=0.0, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=0.0, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000080ff3f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000080ff3f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd03000000000000803f000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd03000000000000803f000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000080ffff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd0300000000000080ffff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd03000000000000803fc00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bd03000000000000803fc00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=0.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=0.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000080ff3f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000080ff3f0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff010000803f000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff010000803f000100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=0, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=0, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000080ffff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff01000080ffff0100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=155, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=155, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff010000803fc00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("bdfbffffff010000803fc00100000000")  # overflow
    assert SmpmUlDeviceEnergy16BStatusInfoData(date_time=timedelta(seconds=1073741823.0), uptime_s=timedelta(seconds=0.0), battery_volts=12.7, temperature=-100, relay_switch=3, relay_is_active=True) == SmpmUlDeviceEnergy16BStatusInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BStatusInfoData.serialize(SmpmUlDeviceEnergy16BStatusInfoData(battery_volts=12.7, date_time=timedelta(seconds=1073741823.0), relay_is_active=True, relay_switch=3, temperature=-100, uptime_s=timedelta(seconds=2147483648.0)))  # noqa: E501



from data_gateway_sdk.protocols.smpm.smpm_ul_device_energy_16b_device_info import SmpmUlDeviceEnergy16BDeviceInfoData
from data_gateway_sdk.utils.buf_ref import BufRef


def test_smpm_ul_device_energy_16b_device_info() -> None:
    case_serialized = bytes.fromhex("befbffffffffffeffbfbf70300000000")
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=562949953421311, device_type=31, firmware_version=127, ktt=255, ktn=63) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=562949953421311, device_type=31, firmware_version=127, ktt=255, ktn=63))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000000000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=0, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=0, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1f0000000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=0, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=0, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be0300000000000000f80f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=0, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=0, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1f00f80f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=0, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=0, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000000000f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=0, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=127, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1f0000f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=0, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=127, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be0300000000000000f8ff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=0, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=127, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1f00f8ff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=0, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=0, ktn=127, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be03000000000000f807000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=255, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=0, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1ff807000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=255, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=0, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be03000000000000f8ff0f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=255, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=0, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1ff8ff0f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=255, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=0, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be03000000000000f807f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=255, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=127, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1ff807f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=255, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=127, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be03000000000000f8ffff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=0, firmware_version=255, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=127, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffff1ff8ffff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=0, firmware_version=255, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=0, firmware_version=255, ktn=127, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e00700000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=0, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=0, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffff0700000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=0, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=0, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e007f80f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=0, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=0, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffff07f80f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=0, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=0, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e00700f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=0, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=127, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffff0700f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=0, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=127, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e007f8ff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=0, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=127, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffff07f8ff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=0, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=0, ktn=127, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e0ff07000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=255, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=0, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffffff07000000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=255, ktt=0, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=0, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e0ffff0f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=255, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=0, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffffffff0f0000000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=255, ktt=511, ktn=0) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=0, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e0ff07f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=255, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=127, ktt=0, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffffff07f00700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=255, ktt=0, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=127, ktt=0, manufacturer_number=1125899906842623))  # noqa: E501
    case_serialized = bytes.fromhex("be030000000000e0ffffff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=0, device_type=63, firmware_version=255, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=127, ktt=511, manufacturer_number=0))  # noqa: E501
    case_serialized = bytes.fromhex("befbffffffffffffffffff0700000000")  # overflow
    assert SmpmUlDeviceEnergy16BDeviceInfoData(manufacturer_number=1125899906842623, device_type=63, firmware_version=255, ktt=511, ktn=127) == SmpmUlDeviceEnergy16BDeviceInfoData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceEnergy16BDeviceInfoData.serialize(SmpmUlDeviceEnergy16BDeviceInfoData(device_type=63, firmware_version=255, ktn=127, ktt=511, manufacturer_number=1125899906842623))  # noqa: E501



import math
import re
from datetime import datetime
from enum import Enum, unique
from typing import Any, Dict, List, Optional, Union
from uuid import UUID

from data_aggregator_sdk.constants.enums import SignalModulation, DownlinkTaskType
from data_aggregator_sdk.integration_message import IntegrationV0MessageMeta, IntegrationV0MessageMetaBSChannelProtocol, \
    IntegrationV0MessageMetaBSHttp
from pydantic import field_validator, ConfigDict, BaseModel
from pydantic_core.core_schema import ValidationInfo

from data_gateway_sdk.errors import DataGatewayBSProtocolParsingError
from data_gateway_sdk.protocols.nero_bs_packet.nero_bs_packet import NeroBsPacket


def hex_is_valid(value: str, hex_length: int = 0) -> bool:
    if hex_length == 0:
        if len(value) % 2 == 0 and re.fullmatch("^#?([A-Fa-f0-9]+)$", value):
            return True
        return False
    else:
        if re.fullmatch("^#?([A-Fa-f0-9]{" + str(hex_length * 2) + "})$", value):
            return True
        return False


class DataNbfi(BaseModel):
    mac: int
    f_ask: bool
    iterator: int
    multi: bool
    system: bool
    payload: str

    @field_validator('mac')
    @classmethod
    def validate_mac(cls, value: int) -> int:
        if 0 <= value < math.pow(2, 32):
            return value
        else:
            raise ValueError(f'Mac must be in range (1, 2^32-1). {value} was given')

    @field_validator('iterator')
    @classmethod
    def validate_iterator(cls, value: int) -> int:
        if 0 <= value < math.pow(2, 32):
            return value
        else:
            raise ValueError(f'Iterator must be in range (0, 2^32-1). {value} was given')

    @field_validator('payload')
    @classmethod
    def validate_payload(cls, value: str) -> str:
        if hex_is_valid(value, 8):
            return value
        else:
            raise ValueError(f'Payload is not a valid HEX. {value} was given')


class DataUnbp(BaseModel):
    mac: int
    ack: bool
    iterator: int
    payload: str

    @field_validator('mac')
    @classmethod
    def validate_mac(cls, value: int) -> int:
        if 0 <= value < math.pow(2, 32):
            return value
        else:
            raise ValueError(f'Mac must be in range (1, 2^32-1). {value} was given')

    @field_validator('iterator')
    @classmethod
    def validate_iterator(cls, value: int) -> int:
        if 0 <= value < math.pow(2, 32):
            return value
        else:
            raise ValueError(f'Iterator must be in range (0, 2^32-1). {value} was given')

    @field_validator('payload')
    @classmethod
    def validate_payload(cls, value: str) -> str:
        if hex_is_valid(value, 4) or hex_is_valid(value, 16) or hex_is_valid(value, 24):  # noqa: E501
            return value
        else:
            raise ValueError(f'Payload is not a valid HEX. {value} was given')


class DataUnbp2(BaseModel):
    mac: int
    payload: str

    @field_validator('mac')
    @classmethod
    def validate_mac(cls, value: int) -> int:
        if 0 <= value < math.pow(2, 32):
            return value
        else:
            raise ValueError(f'Mac must be in range (1, 2^32-1). {value} was given')

    @field_validator('payload')
    @classmethod
    def validate_payload(cls, value: str) -> str:
        hex_length = int(len(value) / 2)
        if hex_is_valid(value, hex_length) and hex_length <= 128:  # noqa: E501
            return value
        else:
            raise ValueError(f'Payload is not a valid HEX. {value} was given')


class SdrInfo(BaseModel):
    freq: int
    freq_channel: int
    sdr: int
    baud_rate: int
    rssi: int
    snr: int

    model_config = ConfigDict(extra="allow")

    @field_validator('freq')
    @classmethod
    def validate_freq(cls, value: int) -> int:
        if value >= 0:
            return value
        else:
            raise ValueError(f'Freq must be more than 0. {value} was given')

    @field_validator('freq_channel', 'baud_rate')
    @classmethod
    def validate_baud_rate(cls, value: int) -> int:
        if value >= 0:
            return value
        else:
            raise ValueError(f'Freq cannot be less than 0. {value} was given')


@unique
class DevicePackageNetworkTypeEnum(Enum):
    downlink = 'downlink'
    uplink = 'uplink'

    def __repr__(self) -> str:
        return f'{type(self).__name__}.{self.name}'


@unique
class DevicePackageProtocolTypeEnum(Enum):
    none = 'none'
    nbfi = 'nbfi'
    unbp = 'unbp'
    unbp2 = 'unbp2'

    def __repr__(self) -> str:
        return f'{type(self).__name__}.{self.name}'


@unique
class DModEnum(Enum):
    DBPSK = 'DBPSK'
    FSK = 'FSK'

    def __repr__(self) -> str:
        return f'{type(self).__name__}.{self.name}'


class DevicePackage(BaseModel):
    footprint: Optional[str] = None
    dt: datetime
    raw: str
    data_nbfi: Optional[DataNbfi] = None
    data_unbp: Optional[DataUnbp] = None
    data_unbp2: Optional[DataUnbp2] = None
    sdr_info: Optional[SdrInfo] = None
    type: DevicePackageNetworkTypeEnum
    protocol: DevicePackageProtocolTypeEnum
    message_id: Optional[int] = None

    @property
    def snr(self) -> int | None:
        if self.sdr_info is None:
            return None
        return self.sdr_info.snr

    @property
    def rssi(self) -> int | None:
        if self.sdr_info is None:
            return None
        return self.sdr_info.rssi

    @property
    def payload_bytes(self) -> bytes:
        if self.protocol is DevicePackageProtocolTypeEnum.unbp:
            assert self.data_unbp
            return bytes.fromhex(self.data_unbp.payload)
        if self.protocol is DevicePackageProtocolTypeEnum.unbp2:
            assert self.data_unbp2
            return bytes.fromhex(self.data_unbp2.payload)
        if self.protocol is DevicePackageProtocolTypeEnum.nbfi:
            assert self.data_nbfi
            return bytes.fromhex(self.data_nbfi.payload)
        raise NotImplementedError(f'not implemented for protocol "{self.type.name}"')

    @property
    def mac(self) -> int:
        if self.protocol is DevicePackageProtocolTypeEnum.unbp:
            assert self.data_unbp
            return self.data_unbp.mac
        if self.protocol is DevicePackageProtocolTypeEnum.unbp2:
            assert self.data_unbp2
            return self.data_unbp2.mac
        if self.protocol is DevicePackageProtocolTypeEnum.nbfi:
            assert self.data_nbfi
            return self.data_nbfi.mac
        raise NotImplementedError(f'not implemented for protocol "{self.type.name}"')

    model_config = ConfigDict(extra="allow")

    @field_validator('raw')
    @classmethod
    def validate_payload(cls, value: str) -> str:
        if hex_is_valid(value):
            return value
        else:
            raise ValueError('Payload is not a valid HEX')

    @field_validator('protocol', mode='before')
    @classmethod
    def validate_protocol(cls, value: Union[str, DevicePackageProtocolTypeEnum], info: ValidationInfo) -> DevicePackageProtocolTypeEnum:
        if value == DevicePackageProtocolTypeEnum.unbp.value or value is DevicePackageProtocolTypeEnum.unbp:
            if info.data.get("data_nbfi", None) is not None:
                raise ValueError(f'data_nbfi for protocol "{value}" must be None. "{type(info.data.get("data_nbfi", None)).__name__}" was given')
            if info.data.get("data_unbp2", None) is not None:
                raise ValueError(f'data_unbp2 for protocol "{value}" must be None. "{type(info.data.get("data_unbp2", None)).__name__}" was given')
            if not isinstance(info.data.get('data_unbp', None), (dict, DataUnbp)):
                raise ValueError(f'data_unbp for protocol "{value}" must be dict. "{type(info.data.get("data_unbp", None)).__name__}" was given')
            if isinstance(value, str):
                return DevicePackageProtocolTypeEnum(value)
            return value
        if value == DevicePackageProtocolTypeEnum.unbp2.value or value is DevicePackageProtocolTypeEnum.unbp2:
            if info.data.get("data_nbfi", None) is not None:
                raise ValueError(f'data_nbfi for protocol "{value}" must be None. "{type(info.data.get("data_nbfi", None)).__name__}" was given')
            if info.data.get("data_unbp", None) is not None:
                raise ValueError(f'data_unbp for protocol "{value}" must be None. "{type(info.data.get("data_unbp", None)).__name__}" was given')
            if not isinstance(info.data.get('data_unbp2', None), (dict, DataUnbp2)):
                raise ValueError(f'data_unbp2 for protocol "{value}" must be dict. "{type(info.data.get("data_unbp2", None)).__name__}" was given')
            if isinstance(value, str):
                return DevicePackageProtocolTypeEnum(value)
            return value
        if value == DevicePackageProtocolTypeEnum.nbfi.value or value is DevicePackageProtocolTypeEnum.nbfi:
            if info.data.get("data_unbp", None) is not None:
                raise ValueError(f'data_unbp for protocol "{value}" must be None. "{type(info.data.get("data_unbp", None)).__name__}" was given')
            if info.data.get("data_unbp2", None) is not None:
                raise ValueError(f'data_unbp2 for protocol "{value}" must be None. "{type(info.data.get("data_unbp2", None)).__name__}" was given')
            if not isinstance(info.data.get('data_nbfi', None), (dict, DataNbfi)):
                raise ValueError(f'data_nbfi for protocol "{value}" must be dict. "{type(info.data.get("data_nbfi", None)).__name__}" was given')
            if isinstance(value, str):
                return DevicePackageProtocolTypeEnum(value)
            return value
        raise ValueError(f'Invalid protocol. "{value}" was given')


class BaseStationEnvironmentInfo(BaseModel):
    dt: datetime
    spectrum: List[float]
    sdr: int
    freq_carrier: float
    freq_delta: float


class BaseStationInfoGeo(BaseModel):
    dt: datetime
    longitude: float = 0.0
    latitude: float = 0.0
    actual: bool


class SdrConfigData(BaseModel):
    enable: bool
    dds_freq: int
    dmod: DModEnum
    baud_rate: int
    unions: int

    @field_validator('dmod', mode="before")
    @classmethod
    def validate_dmod(cls, value: Union[DModEnum, str]) -> DModEnum:
        return DModEnum(re.compile(r'[^a-zA-Z0-9_-]').sub('', value).upper()) if isinstance(value, str) else value


class SdrConfig(BaseModel):
    freq: int
    sdr: List[SdrConfigData]


class BaseStationInfo(BaseModel):
    geo: BaseStationInfoGeo
    uptime_s: int
    sdr_config: Optional[SdrConfig] = None

    @field_validator('uptime_s')
    @classmethod
    def validate_mac(cls, value: int) -> int:
        if value >= 0:
            return value
        else:
            raise ValueError('Uptime_s cannot be less than 0')


class BaseStationAdditionalInfo(BaseModel):
    id: int  # SERIAL NUMBER
    version: Optional[str] = None
    # TODO: it's was just example additional info
    # any: str


@unique
class BaseStationLogsTypeEnum(Enum):
    emerg = 'emerg'
    alert = 'alert'
    critical = 'critical'
    error = 'error'
    warning = 'warning'
    notice = 'notice'
    debug = 'debug'
    info = 'info'

    def __repr__(self) -> str:
        return f'{type(self).__name__}.{self.name}'


@unique
class BsDownlinkCommandNameEnum(Enum):
    sync_downlink_tasks = "sync_downlink_tasks"
    add_downlink_task = "add_downlink_task"
    delete_downlink_task = "delete_downlink_tasks"
    get_active_downlink_task_ids = "get_active_downlink_task_ids"

    def __repr__(self) -> str:
        return f'{type(self).__name__}.{self.name}'


class DataAddDownlinkTaskResponseInvalidTasks(BaseModel):
    id: UUID
    reason: str


class DataAddDownlinkTaskResponse(BaseModel):
    accepted_ids: List[UUID]
    already_exist_ids: List[UUID]
    invalid: List[DataAddDownlinkTaskResponseInvalidTasks]


class DataDeleteDownlinkTaskResponse(BaseModel):
    deleted_ids: List[UUID]


class DataGetActiveDownlinkTaskIDSResponse(BaseModel):
    active_ids: List[UUID]


class BsDownlinkTaskExecutionsSignalParams(BaseModel):
    freq: int
    power: int
    baudrate: int
    modulation: SignalModulation


class BsDownlinkTaskExecutionsDataUnbpMessage(BaseModel):
    payload: Optional[str] = None


class BsDownlinkTaskExecutionsDataTimeSync(BaseModel):
    dt: Optional[datetime] = None


class BaseStationLogs(BaseModel):
    type: BaseStationLogsTypeEnum
    mdl: Optional[str] = None
    dt: datetime
    text: str

    @field_validator('mdl', 'text')
    @classmethod
    def validate_mac(cls, value: str) -> str:
        return value.strip()


class BsDownlinkCommandRequests(BaseModel):
    command: BsDownlinkCommandNameEnum


class BsDownlinkCommandResponse(BaseModel):
    dt: datetime
    id: UUID
    status_ok: bool
    command: BsDownlinkCommandNameEnum
    data_add_downlink_task: Optional[DataAddDownlinkTaskResponse] = None
    data_delete_downlink_tasks: Optional[DataDeleteDownlinkTaskResponse] = None
    data_get_active_downlink_task_ids: Optional[DataGetActiveDownlinkTaskIDSResponse] = None


class BsDownlinkTaskExecutions(BaseModel):
    task_id: UUID
    execution_start_dt: datetime
    execution_end_dt: datetime
    status_ok: bool
    status_message: str
    mac: int
    signal_params: Optional[BsDownlinkTaskExecutionsSignalParams] = None
    type: DownlinkTaskType
    data_unbp_message: Optional[BsDownlinkTaskExecutionsDataUnbpMessage] = None
    data_time_sync: Optional[BsDownlinkTaskExecutionsDataTimeSync] = None


class BsDownlinkTaskDeletionAsync(BaseModel):
    task_id: UUID
    dt: datetime
    status_message: str


class HttpV0NeroBsPacket(NeroBsPacket):
    data: List[DevicePackage]
    base_station_environment_info: Optional[List[BaseStationEnvironmentInfo]] = None
    base_station_info: BaseStationInfo
    base_station_additional_info: BaseStationAdditionalInfo
    base_station_logs: Optional[List[BaseStationLogs]] = None
    commands_requests: Optional[List[BsDownlinkCommandRequests]] = None
    commands_responses: Optional[List[BsDownlinkCommandResponse]] = None
    downlink_task_executions: Optional[List[BsDownlinkTaskExecutions]] = None
    downlink_task_deleting_async: Optional[List[BsDownlinkTaskDeletionAsync]] = None

    def to_integration_meta(self, *, base_station_api_user_id: UUID, packet: DevicePackage, **kwargs: Any) -> IntegrationV0MessageMeta:  # type: ignore
        assert packet.sdr_info is not None
        return IntegrationV0MessageMeta(
            bs_http=IntegrationV0MessageMetaBSHttp(
                freq=packet.sdr_info.freq,
                freq_channel=packet.sdr_info.freq_channel,
                sdr=packet.sdr_info.sdr,
                baud_rate=packet.sdr_info.baud_rate,
                rssi=packet.sdr_info.rssi,
                snr=packet.sdr_info.snr,
                mac=packet.mac,
                station_id=base_station_api_user_id,
                station_serial_number=self.base_station_additional_info.id,
                dt_detected=packet.dt,
                dt_published=packet.dt,
                channel_protocol=IntegrationV0MessageMetaBSChannelProtocol(packet.protocol.value),
            ),
        )

    @classmethod
    def parse(cls, data: Dict[str, Any], **kwargs: Any) -> 'HttpV0NeroBsPacket':
        try:
            return HttpV0NeroBsPacket(
                **data,
            )
        except Exception as e:  # noqa: B902
            raise DataGatewayBSProtocolParsingError('invalid payload', e)



from datetime import datetime, tzinfo
from enum import IntEnum, unique
from typing import List, Any, Dict

from data_aggregator_sdk.integration_message import IntegrationV0MessageData

from data_gateway_sdk.utils.buf_ref import BufRef
from data_gateway_sdk.utils.packet import Packet


# PACKET (64 bits)   smpm_ul_device_dl_answer
#
# RESULT int:        540431955267682563
# RESULT bin:  MSB   00000111 01111111 11111111 11111111 11111111 00000000 00000001 00000011   LSB
# RESULT hex:  LE    03 01 00 ff ff ff 7f 07
#
# name                  type                                      size  value(int)                                                        data(bits)
# --------------------  ----------------------------------------  ----  ----------  ----------------------------------------------------------------
# packet_type_id.0.VAL  u7                                           7           3                                                           0000011
# packet_type_id.0.DFF  bool                                         1           0                                                          0
# downlink_packet_id    SmpmUlDeviceDlAnswerDataDownlinkPacketId    16           1                                          0000000000000001
# downlink_packet_crc   u32                                         32  2147483647          01111111111111111111111111111111
# answer_packets_count  u4                                           4           7      0111
# RESERVED              u4                                           4           0  0000


@unique
class SmpmUlDeviceDlAnswerDataDownlinkPacketId(IntEnum):
    GET_ECHO = 1
    SET_CLOCK = 2
    GET_DATA_SHORT = 128
    GET_DATA_LONG = 129
    SET_REGULAR_DATA_SENDING = 150
    SET_RELAY = 170


class SmpmUlDeviceDlAnswerData(Packet):
    downlink_packet_id: SmpmUlDeviceDlAnswerDataDownlinkPacketId
    downlink_packet_crc: int
    answer_packets_count: int

    def serialize(self) -> bytes:
        data = self
        result = 0
        size = 0
        result |= ((3) & (2 ** (7) - 1)) << size
        size += 7
        result |= ((0) & (2 ** (1) - 1)) << size
        size += 1
        assert isinstance(data.downlink_packet_id, SmpmUlDeviceDlAnswerDataDownlinkPacketId)
        result |= ((data.downlink_packet_id.value) & (2 ** (16) - 1)) << size
        size += 16
        assert isinstance(data.downlink_packet_crc, int)
        assert 0 <= data.downlink_packet_crc <= 4294967295
        result |= ((data.downlink_packet_crc) & (2 ** (32) - 1)) << size
        size += 32
        assert isinstance(data.answer_packets_count, int)
        assert 0 <= data.answer_packets_count <= 15
        result |= ((data.answer_packets_count) & (2 ** (4) - 1)) << size
        size += 4
        return result.to_bytes(8, "little")

    @classmethod
    def parse(cls, buf: BufRef) -> 'SmpmUlDeviceDlAnswerData':
        result__el_tmp1: Dict[str, Any] = dict()
        if 3 != buf.shift(7):
            raise ValueError("packet_type_id: buffer doesn't match value")
        if 0 != buf.shift(1):
            raise ValueError("packet_type_id: buffer doesn't match value")
        result__el_tmp1["downlink_packet_id"] = SmpmUlDeviceDlAnswerDataDownlinkPacketId(buf.shift(16))
        result__el_tmp1["downlink_packet_crc"] = buf.shift(32) + 0
        result__el_tmp1["answer_packets_count"] = buf.shift(4) + 0
        result = SmpmUlDeviceDlAnswerData(**result__el_tmp1)
        buf.shift(4)
        return result

    def to_integration_data(self, received_at: datetime, device_tz: tzinfo, **kwargs: Any) -> List[IntegrationV0MessageData]:
        return [
            IntegrationV0MessageData(
                dt=received_at,
                # TODO: handle packet info if it will be necessary
            ),
        ]



import random
import typing
from datetime import timedelta

from data_gateway_sdk.utils.buf_ref import BufRef
from data_gateway_sdk.utils.packet import Packet


# PACKET (64 bits)   smpm_dl_device_energy_8b_set_clock
#
# RESULT int:        72057044282113794
# RESULT bin:  MSB   00000000 11111111 11111111 01111111 11111111 11111111 11111111 00000010   LSB
# RESULT hex:  LE    02 ff ff ff 7f ff ff 00
#
# name                          type       size  value(int)                                                        data(bits)
# ----------------------------  ---------  ----  ----------  ----------------------------------------------------------------
# packet_type_id.0.VAL          u7            7           2                                                           0000010
# packet_type_id.0.DFF          bool          1           0                                                          0
# time                          timedelta    32  2147483647                          01111111111111111111111111111111
# time_zone_offset_s            timedelta    17       65535         01111111111111111
# time_zone_offset_is_negative  bool          1           0        0
# RESERVED                      u6            6           0  000000
# Reserved space is filled with random bits in DOWNLINK packets !


class SmpmDlDeviceEnergy8BSetClockData(Packet):
    time: timedelta
    time_zone_offset_s: timedelta
    time_zone_offset_is_negative: bool

    def serialize(self) -> bytes:
        data = self
        result = 0
        size = 0
        result |= ((2) & (2 ** (7) - 1)) << size
        size += 7
        result |= ((0) & (2 ** (1) - 1)) << size
        size += 1
        isinstance(data.time, (int, timedelta))
        time_tmp1 = int(data.time.total_seconds() // 1 if isinstance(data.time, timedelta) else data.time // 1)
        assert 0 <= time_tmp1 <= 4294967295
        result |= ((time_tmp1) & (2 ** (32) - 1)) << size
        size += 32
        isinstance(data.time_zone_offset_s, (int, timedelta))
        time_zone_offset_s_tmp2 = int(data.time_zone_offset_s.total_seconds() // 1 if isinstance(data.time_zone_offset_s, timedelta) else data.time_zone_offset_s // 1)
        assert 0 <= time_zone_offset_s_tmp2 <= 131071
        result |= ((time_zone_offset_s_tmp2) & (2 ** (17) - 1)) << size
        size += 17
        assert isinstance(data.time_zone_offset_is_negative, bool)
        result |= ((int(data.time_zone_offset_is_negative)) & (2 ** (1) - 1)) << size
        size += 1
        result |= ((random.getrandbits(6)) & (2 ** 6 - 1)) << size
        size += 6
        return result.to_bytes(8, "little")

    @classmethod
    def parse(cls, buf: BufRef) -> 'SmpmDlDeviceEnergy8BSetClockData':
        result__el_tmp3: typing.Dict[str, typing.Any] = dict()
        if 2 != buf.shift(7):
            raise ValueError("packet_type_id: buffer doesn't match value")
        if 0 != buf.shift(1):
            raise ValueError("packet_type_id: buffer doesn't match value")
        result__el_tmp3["time"] = timedelta(seconds=buf.shift(32) * 1)
        result__el_tmp3["time_zone_offset_s"] = timedelta(seconds=buf.shift(17) * 1)
        result__el_tmp3["time_zone_offset_is_negative"] = bool(buf.shift(1))
        result = SmpmDlDeviceEnergy8BSetClockData(**result__el_tmp3)
        buf.shift(6)
        return result
